@@ -1,4 +1,4 @@
-# $Id: Event.pm 2219 2007-08-19 00:54:55Z rcaputo $
+# $Id: Event.pm,v 1.41 2004/11/16 07:54:11 teknikill Exp $
 
 # Event.pm event loop bridge for POE::Kernel.
 
@@ -7,26 +7,8 @@ package POE::Loop::Event;
 
 use strict;
 
-# Include common signal handling.  Signals should be safe now, and for
-# some reason Event isn't dispatching SIGCHLD to me circa POE r2084.
-use POE::Loop::PerlSignals;
-
 use vars qw($VERSION);
-$VERSION = do {my($r)=(q$Revision: 2219 $=~/(\d+)/);sprintf"1.%04d",$r};
-
-=for poe_tests
-
-sub skip_tests {
-  return "Event tests require the Event module" if (
-    do { eval "use Event"; $@ }
-  );
-  my $test_name = shift;
-  if ($test_name eq "k_signals_rerun" and $^O eq "MSWin32") {
-    return "This test crashes Perl when run with Tk on $^O";
-  }
-}
-
-=cut
+$VERSION = do {my@r=(q$Revision: 1.41 $=~/\d+/g);sprintf"%d."."%04d"x$#r,@r};
 
 # Everything plugs into POE::Kernel.
 package POE::Kernel;
@@ -40,18 +22,25 @@ my %signal_watcher;
 #------------------------------------------------------------------------------
 # Loop construction and destruction.
 
+sub _our_event_exception_handler {
+  my ($event, $message) = @_;
+  warn $message;
+  exit 1;
+}
+
 sub loop_initialize {
   my $self = shift;
 
+  $Event::DIED = \&_our_event_exception_handler;
+
   $_watcher_timer = Event->timer(
     cb     => \&_loop_event_callback,
     after  => 0,
+    parked => 1,
   );
 }
 
 sub loop_finalize {
-  my $self = shift;
-
   foreach my $fd (0..$#fileno_watcher) {
     next unless defined $fileno_watcher[$fd];
     foreach my $mode (MODE_RD, MODE_WR, MODE_EX) {
@@ -60,13 +49,92 @@ sub loop_finalize {
       ) if defined $fileno_watcher[$fd]->[$mode];
     }
   }
+}
 
-  $self->loop_ignore_all_signals();
+#------------------------------------------------------------------------------
+# Signal handlers/callbacks.
+
+sub _loop_signal_handler_generic {
+  if (TRACE_SIGNALS) {
+    POE::Kernel::_warn "<sg> Enqueuing generic SIG$_[0] event";
+  }
+
+  $poe_kernel->_data_ev_enqueue(
+    $poe_kernel, $poe_kernel, EN_SIGNAL, ET_SIGNAL, [ $_[0]->w->signal ],
+    __FILE__, __LINE__, undef, time(),
+  );
+}
+
+sub _loop_signal_handler_pipe {
+  if (TRACE_SIGNALS) {
+    POE::Kernel::_warn "<sg> Enqueuing PIPE-like SIG$_[0] event";
+  }
+
+  $poe_kernel->_data_ev_enqueue(
+    $poe_kernel->get_active_session(), $poe_kernel,
+    EN_SIGNAL, ET_SIGNAL, [ $_[0]->w->signal ],
+    __FILE__, __LINE__, undef, time(),
+  );
+}
+
+sub _loop_signal_handler_child {
+  if (TRACE_SIGNALS) {
+    POE::Kernel::_warn "<sg> Enqueuing CHLD-like SIG$_[0] event";
+  }
+
+  $poe_kernel->_data_ev_enqueue(
+    $poe_kernel, $poe_kernel, EN_SCPOLL, ET_SCPOLL, [ ],
+    __FILE__, __LINE__, undef, time(),
+  );
 }
 
 #------------------------------------------------------------------------------
 # Signal handler maintenance functions.
 
+sub loop_watch_signal {
+  my ($self, $signal) = @_;
+
+  # Child process has stopped.
+  if ($signal eq 'CHLD' or $signal eq 'CLD') {
+
+    # Begin constant polling loop.  Only start it on CHLD or on CLD if
+    # CHLD doesn't exist.
+    $SIG{$signal} = 'DEFAULT';
+    $self->_data_ev_enqueue(
+      $self, $self, EN_SCPOLL, ET_SCPOLL, [ ],
+      __FILE__, __LINE__, undef, time() + 1,
+    ) if $signal eq 'CHLD' or not exists $SIG{CHLD};
+
+    return;
+  }
+
+  # Broken pipe.
+  if ($signal eq 'PIPE') {
+    $signal_watcher{$signal} = Event->signal(
+      signal => $signal,
+      cb     => \&_loop_signal_handler_pipe
+    );
+    return;
+  }
+
+  # Event doesn't like watching nonmaskable signals.
+  return if $signal eq 'KILL' or $signal eq 'STOP';
+
+  # Everything else.
+  $signal_watcher{$signal} = Event->signal(
+    signal => $signal,
+    cb     => \&_loop_signal_handler_generic
+  );
+}
+
+sub loop_ignore_signal {
+  my ($self, $signal) = @_;
+  if (defined $signal_watcher{$signal}) {
+    $signal_watcher{$signal}->stop();
+    delete $signal_watcher{$signal};
+  }
+}
+
 sub loop_attach_uidestroy {
   # does nothing
 }
@@ -76,20 +144,17 @@ sub loop_attach_uidestroy {
 
 sub loop_resume_time_watcher {
   my ($self, $next_time) = @_;
-  ($_watcher_timer and $next_time) or return;
   $_watcher_timer->at($next_time);
   $_watcher_timer->start();
 }
 
 sub loop_reset_time_watcher {
   my ($self, $next_time) = @_;
-  $_watcher_timer or return;
   $self->loop_pause_time_watcher();
   $self->loop_resume_time_watcher($next_time);
 }
 
 sub loop_pause_time_watcher {
-  $_watcher_timer or return;
   $_watcher_timer->stop();
 }
 
@@ -162,6 +227,24 @@ sub _loop_event_callback {
   $self->_data_ev_dispatch_due();
   $self->_test_if_kernel_is_idle();
 
+  # Register the next timed callback if there are events left.
+
+  my $next_time = $self->get_next_event_time();
+  if (defined $next_time) {
+    $_watcher_timer->at($next_time);
+    $_watcher_timer->start();
+
+    # POE::Kernel's signal polling loop always keeps oe event in the
+    # queue.  We test for an idle kernel if the queue holds only one
+    # event.  A more generic method would be to keep counts of user
+    # vs. kernel events, and GC the kernel when the user events drop
+    # to 0.
+
+    if ($self->_data_ses_count() == 1) {
+      $self->_test_if_kernel_is_idle();
+    }
+  }
+
   # Transferring control back to Event; this is idle time.
   $last_time = time() if TRACE_STATISTICS;
 }
@@ -195,19 +278,15 @@ sub _loop_select_callback {
 # The event loop itself.
 
 sub loop_do_timeslice {
-  Event::one_event();
+  die "doing timeslices currently not supported in the Event loop";
 }
 
 sub loop_run {
-  my $self = shift;
-  while ($self->_data_ses_count()) {
-    $self->loop_do_timeslice();
-  }
+  Event::loop();
 }
 
 sub loop_halt {
   $_watcher_timer->stop();
-  $_watcher_timer = undef;
   Event::unloop_all(0);
 }
 
@@ -239,6 +318,3 @@ Please see L<POE> for more information about authors, contributors,
 and POE's licensing.
 
 =cut
-
-# rocco // vim: ts=2 sw=2 expandtab
-# TODO - Redocument.
